/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Automount.h"
#include "QDebug.h"
#include "VolumeMountGui.h"
#include "Util.h"

#include <glibmm/i18n.h>
#include <assert.h>

//==============================================================================

Automount::Automount( 	std::list<VolumeGui*>& lstVolumeGui,
						Glib::Mutex& omtxVolumeGuiLst)
: glstVolumeGuiAct(lstVolumeGui),
  gomtxVolumeGuiLstAct(omtxVolumeGuiLst)
{
	gbStop = false;
	
	// create local list
	Glib::Mutex::Lock lock(omtxVolumeGuiLst);
	
	for (   std::list<VolumeGui*>::iterator ite = lstVolumeGui.begin();
			ite != lstVolumeGui.end();
			++ite)
	{
		VolumeMountGui* poVolumeGui = (VolumeMountGui*)(*ite);
		VolumeConfig &oVolumeConfig = poVolumeGui->getVolumeConfig();
		
		poVolumeGui->signal_mounted().connect(
				sigc::bind<1>(
					sigc::mem_fun(*this, &Automount::onMounted), poVolumeGui));
										
		if (oVolumeConfig.isAutomount())
		{
			if (poVolumeGui->lockThread())
			{
				bool bMounted = false;
				bool bInUse = poVolumeGui->isInUse(&bMounted);
				
				if (!bInUse)
					glstVolumeGui.push_back(poVolumeGui);
				
				poVolumeGui->lockThread(false);
			}
		}
	}
}

//==============================================================================

Automount::~Automount()
{
}

//==============================================================================

void Automount::getVolumes(	std::vector<VolumeGui*> *pvecToMount, 
							std::vector<VolumeGui*> *pvecToMountErr)
{
	bool bToMount = false;
	bool bToMountErr = false;
	bool bInUse = false;
	bool bMounted = false;
	
	std::vector<VolumeGui*> vecVolumeGui;
	
	// get list into vector
	for (   std::list<VolumeGui*>::iterator ite = glstVolumeGui.begin();
			ite != glstVolumeGui.end();
			++ite)
	{
		vecVolumeGui.push_back(*ite);
	}
	
	// clear input
	pvecToMount->clear();
	pvecToMountErr->clear();
	
	// process vec tomount
	for (   std::vector<VolumeGui*>::size_type xi = 0;
			xi<vecVolumeGui.size();
			++xi)
	{
		VolumeMountGui* poVolumeGui = (VolumeMountGui*)vecVolumeGui[xi];
		
		if (!isVolumeGuiFound(poVolumeGui))
		{
			continue;
		}
		
		VolumeConfig &oVolumeConfig = poVolumeGui->getVolumeConfig();
		
		bToMount = false;
		bToMountErr = false;
		
		if (!poVolumeGui->lockThread())
			continue;
			
		bMounted = false;
		bInUse = poVolumeGui->isInUse(&bMounted);
			
		if (oVolumeConfig.isAutomount() && (!bInUse || !bMounted))
		{
			if (oVolumeConfig.getMountData())
			{
				if (Util::isFileExist(oVolumeConfig.getData().getSrc())
					&& Util::isFileExist(oVolumeConfig.getMountData()->getDir()))
				{
					if (oVolumeConfig.getCryptData()
						&& oVolumeConfig.getCryptData()->getIsEncrypted()
						&& !oVolumeConfig.getCryptData()->getKeyFile().empty())
					{
						if (Util::isFileExist(oVolumeConfig.getCryptData()->getKeyFile()))
							bToMount = true;
					}
					else
						bToMount = true;
				}
				else
					bToMountErr = true;
			}
		}
		
		if (bToMount)
			pvecToMount->push_back(poVolumeGui);
		else if (bToMountErr)
			pvecToMountErr->push_back(poVolumeGui);
			
		poVolumeGui->lockThread(false);
	}
}

//==============================================================================

bool Automount::isVolumeGuiFound(VolumeGui* poVolumeGui)
{
	Glib::Mutex::Lock lock(gomtxVolumeGuiLstAct);
	
	for (   std::list<VolumeGui*>::iterator ite = glstVolumeGuiAct.begin();
			ite != glstVolumeGuiAct.end();
			++ite)
	{
		VolumeMountGui* poVolumeGuiLst = (VolumeMountGui*)(*ite);
		
		if (poVolumeGuiLst == poVolumeGui)
			return true;
	}
	
	return false;
}

//==============================================================================

void Automount::onMounted(bool bSuccess, VolumeMountGui* poVolumeGui)
{
	glstVolumeGui.remove(poVolumeGui);
	
	if (!gbStop)
		start();
}

//==============================================================================

void Automount::start()
{
	std::vector<VolumeGui*> vecToMount;
	std::vector<VolumeGui*> vecToMountErr;
	
	getVolumes(&vecToMount, &vecToMountErr);
	
#ifdef QDEBUG
	debug_message(DEBUG_INFO, "Start listing volumes in Automount");
	
	for (   std::list<VolumeGui*>::iterator ite = glstVolumeGui.begin();
			ite != glstVolumeGui.end();
			++ite)
	{
		VolumeMountGui* poVolumeGui = (VolumeMountGui*)(*ite);
		
		if (isVolumeGuiFound(poVolumeGui))
			debug_message(DEBUG_INFO, "Automount: %s", 
				poVolumeGui->getVolumeConfig().getData().getLabel().c_str());
	}
		
	for (unsigned int xi=0; xi<vecToMount.size(); ++xi)
		debug_message(DEBUG_INFO, "Mounting: %s",
			vecToMount[xi]->getVolumeConfig().getData().getLabel().c_str());
		
	for (unsigned int xi=0; xi<vecToMountErr.size(); ++xi)
		debug_message(DEBUG_INFO, "Cannot mount: %s",
			vecToMountErr[xi]->getVolumeConfig().getData().getLabel().c_str());

	debug_message(DEBUG_INFO, "End of listing volumes in Automount");
#endif
	
	for (   std::vector<VolumeGui*>::size_type xi = 0;
			xi<vecToMount.size();
			++xi)
	{
		VolumeMountGui* poVolumeGui = (VolumeMountGui*)vecToMount[xi];
				
		poVolumeGui->mount();
	}
}

//==============================================================================

void Automount::stop()
{
	gbStop = true;
}

//==============================================================================
