/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "EnlargeVolumePline.h"
#include "QDebug.h"
#include "../Util.h"

#include "MountPline.h"

#include "CloneData.h"
#include "CopyFilter.h"
#include "CryptFilter.h"
#include "EnlargeVolumeFilter.h"
#include "LoopFilter.h"
#include "MountFilter.h"
#include "TrueFilter.h"

#include <iostream>
#include <iomanip>
#include <glibmm/i18n.h>
#include <assert.h>

namespace Pline
{

//==============================================================================

EnlargeVolumePline::EnlargeVolumePline( const eDataMode eDataModeItem,
										VolumeConfig *poVolumeConfig,
										const unsigned long long ullEnlargeSize)
: ProcessBasePline(eDataModeItem, poVolumeConfig)
{
	assert(goVolumeConfig.getMountData());
	
	gfCurrentSpeed = 0;
	giPctDone = 0;
	giCopyPctDone = 0;
	
	gbCopying = false;
	gbEjecting = false;
	gbMounting = false;
	gbEnlarging = false;
	
	gpoMntAfterCpy = 0;
	
	gpoEnlargeFilter = 0;
	gpoUnMountFilter = 0;
	gpoMountFilter = 0;
	
	// init mount pline
	gpoMountPline = new MountPline(eDataModeItem, poVolumeConfig);
	gpoMountPline->signal_req_data().connect(
			sigc::mem_fun(*this, &EnlargeVolumePline::onMountPlineReqData));
		
	//
	// Copy
	//
	CopyFilter *poCopyFilter = new CopyFilter;
			
	// set data from config
	CloneData* poCloneData = new CloneData(goVolumeConfig.getData());
	poCloneData->setSrc("/dev/urandom");
	poCloneData->setDest(goVolumeConfig.getData().getSrc());
	
	// set filter data
	poCopyFilter->setSrcData(poCloneData);
	poCopyFilter->setLimit(ullEnlargeSize);
	
	// set filter run mode
	poCopyFilter->setRunMode(CopyFilter::kRunCopyEnlargeExpand);
	
	// set other data
	poCopyFilter->setCurrentSpeed(&gfCurrentSpeed);
	poCopyFilter->setETA(&gstETA);
	poCopyFilter->setPctDone(&giCopyPctDone);
	
	// add filter to pline
	addFilter(poCopyFilter);
	
	//
	// Mount after copy
	// 
	gpoMntAfterCpy = new TrueFilter;
	addFilter(gpoMntAfterCpy);
	
	//
	// Unmount dir
	// 
	gpoUnMountFilter = new MountFilter;
	
	gpoUnMountFilter->setBasePathSrc(&goVolumeConfig.getData());
	
	gpoUnMountFilter->setRunMode(MountBaseFilter::kRunDetach);
	
	addFilter(gpoUnMountFilter);
	
	//
	// Enlarge
	//
	gpoEnlargeFilter = new EnlargeVolumeFilter;
	
	addFilter(gpoEnlargeFilter);
	
	//
	// Mount dir
	//
	gpoMountFilter = new MountFilter;
	
	gpoMountFilter->setBasePathSrc(&goVolumeConfig.getData());
	
	gpoMountFilter->setRunMode(MountBaseFilter::kRunAttach);
	
	addFilter(gpoMountFilter);
}

//==============================================================================

EnlargeVolumePline::~EnlargeVolumePline()
{
	delete gpoMountPline;
}

//==============================================================================

void EnlargeVolumePline::askData()
{
	(gpoReqFilter) ?
		ProcessBasePline::askData() : gpoMountPline->askData();
}

//==============================================================================

const Glib::ustring EnlargeVolumePline::getInfoMessage()
{
	Glib::ustring stMsg = _("Enlarging volume...");
	
	if (gbCopying)
	{
		// Copying
		Glib::ustring stSpeed = 
			Glib::ustring::format(std::fixed, std::setprecision(2), gfCurrentSpeed);
		
		stMsg = Util::compose(	_("Extending disk image at %sM/s, %d%% done. ETA %s..."),
								stSpeed.c_str(),
								giCopyPctDone,
								(gstETA.empty()) ? _("inf") : gstETA.c_str());
		
		int iPctDone = ((float)giCopyPctDone/(float)100)*(float)60;
		giPctDone = iPctDone;
	}
	else if (gbEjecting)
	{
		giPctDone = 70;
		stMsg = _("Ejecting volume...");
	}
	else if (gbEnlarging)
	{
		giPctDone = 80;
		stMsg = _("Resizing filesystem, please wait...");
	}
	else if (gbMounting)
	{
		giPctDone = 90;
		stMsg = _("Mounting volume...");
	}
	
	return stMsg;
}

//==============================================================================

bool EnlargeVolumePline::onFilterPreRun(Filter *poFilter)
{
	gbCopying = (poFilter->getFilterName() == "Copy");
	gbEjecting = (poFilter == gpoUnMountFilter);
	gbMounting = (poFilter == gpoMountFilter);
	gbEnlarging = (poFilter == gpoEnlargeFilter);
	
	if (poFilter == gpoUnMountFilter
		|| poFilter == gpoMountFilter)
	{
		if (gpoMountPline->getVolumeConfig()->getMountData()->getFSys() == "xfs")
			return false;
	}
	
	return ProcessBasePline::onFilterPreRun(poFilter);
}

//==============================================================================

bool EnlargeVolumePline::onFilterRun(Filter *poFilter, bool bHasRunWithErrors)
{
	if (poFilter == gpoMntAfterCpy)
	{
		gpoMountPline->setRunMode(MountPline::kRunMount);
		
		if (!gpoMountPline->run())
		{
			gtLastError = gpoMountPline->getLastError();
			
			return false;
		}
		else
		{
			gpoUnMountFilter->setSrcData(gpoMountPline->getVolumeConfig()->getMountData());
			gpoMountFilter->setSrcData(gpoMountPline->getVolumeConfig()->getMountData());
			gpoEnlargeFilter->setSrcData(gpoMountPline->getVolumeConfig()->getMountData());
		}
	}
	
	return Pline::onFilterRun(poFilter, bHasRunWithErrors);
}

//==============================================================================

void EnlargeVolumePline::onMountPlineReqData()
{
	m_signal_req_data.emit();
}

//==============================================================================

bool EnlargeVolumePline::run()
{	
	initError();
	
	// manage completed signal locally
	Pline::skipCompletedSignal(true);

	bool bMountPlineInUse = gpoMountPline->isInUse();
	
	// eject mount pline
	if (bMountPlineInUse)
	{
		gpoMountPline->setRunMode(MountPline::kRunEject);
		
		if (!gpoMountPline->run())
		{
			gtLastError.stErrMsg = _("Could not eject volume.");
			
			m_signal_completed(false);
			return false;
		}
	}
	
	bool bRet = Pline::run();
	
	if (!bRet)
	{
		gpoMountPline->setRunMode(MountPline::kRunEject);
		gpoMountPline->run();
	}
	
	m_signal_completed(bRet);
	
	return bRet;
}

//==============================================================================

} // Pline	
