/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "MountPline.h"
#include "../QDebug.h"
#include "../Util.h"
#include "MountUtil.h"
#include "CryptUtil.h"

// filters
#include "CryptFilter.h"
#include "LoopFilter.h"
#include "MountFilter.h"
#include "PermFilter.h"
#include "TrashFilter.h"

#include <assert.h>
#include <glibmm/i18n.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

namespace Pline
{

#define PLINE_NOTCREATED_ERR Glib::ustring(_("Could not retrieve device information, are you root?"))

//==============================================================================

MountPline::MountPline( const eDataMode eDataModeItem,
						VolumeConfig *poVolumeConfig)
: Pline(eDataModeItem, poVolumeConfig)
{	
	gpoPermFilter = 0;
	gpoTrashFilter = 0;
	
	geRunMode = kNoRunMode;
	geLastRunMode = kNoRunMode;
	geCurrRunMode = kNoRunMode;
	
	gbHasProbedFilters = false;
	gbProbeError = false;
	gbDumpStatus = false;
	gbEncrypted = false;
	
	initDiskUsage();
	
	gpoVolumeConfig = new VolumeConfig;
	*gpoVolumeConfig = goVolumeConfig;
	
	createVolumeConfigPline(gpoVolumeConfig);
}

//==============================================================================

MountPline::~MountPline()
{
	if (gpoVolumeConfig)
		delete gpoVolumeConfig;
}

//==============================================================================

bool MountPline::calcDiskUsage()
{		
	bool bMounted = false;
	
	std::vector<Filter*> vecFilter = Pline::getFilters();

	debug_message(  DEBUG_PLINE,  "Src[%s] calculating disk usage.", 
					gpoVolumeConfig->getData().getSrc().c_str());
										
	initError();
	
	initDiskUsage();
	
	gtLastError.bIsFilterError = false;
	
	if (!isInUse(&bMounted) || !bMounted)
	{
		gtLastError.stErrMsg = _("Volume not mounted.");
		
		return false;
	}
			
	if (vecFilter.size() == 0)
	{
		gtLastError.bIsFilterError = false;
		
		// test src
		Glib::ustring stSrc = gpoVolumeConfig->getData().getSrc();
		
		Util::isFileExist(stSrc, &gtLastError.stErrMsg);
		
		return false;
	}
	
	for (   std::vector<Filter*>::size_type xi = 0;
			xi < vecFilter.size();
			++xi)
	{
		if (vecFilter[xi] == gpoPermFilter
			|| vecFilter[xi] == gpoTrashFilter)
		{
			continue;
		}
			
		MountBaseFilter *poFilter = (MountBaseFilter*)vecFilter[xi];
		
		if (poFilter->getFilterName() == "Mount")
		{
			MountData *poMountData = (MountData*)poFilter->getSrcData();
			
			if (!poMountData)
				return false;
				
			if (MountUtil::getDiskUsage(poMountData->getDir(),
										&gtDiskUsage.stDiskSize,
										&gtDiskUsage.stSpaceUsed,
										&gtDiskUsage.stSpaceAvailable,
										&gtDiskUsage.iUsedSpacePct, 
										&gtLastError.stErrMsg))
			{
				gtDiskUsage.bValid = true;
				
				return true;
			}
		}
	}
	
	return false;
}

//==============================================================================

void MountPline::createVolumeConfigPline(VolumeConfig *poVolumeConfig)
{
	bool bPlineCreated = false;
	
	eMountType eMountTypeItem;
	
	MountData   *poMountData = 0;
	LoopData	*poLoopData = 0;
	CryptData   *poCryptData = 0;
	
	MountFilter *poMountFilter = 0;
	LoopFilter  *poLoopFilter = 0;
	CryptFilter	*poCryptFilter = 0;
	
	initError();
	
	gpoPermFilter = 0;
	gpoTrashFilter = 0;
	
	// check data
	if (!poVolumeConfig->getDataPtr()
		|| !poVolumeConfig->getMountData())
	{
		gtLastError.bIsFilterError = false;
		gtLastError.stErrMsg = _("Volume configuration unknown.");
		
		return;
	}
				
	switch ((eMountTypeItem = getMountType(poVolumeConfig)))
	{
		case kMountSimple:
			// new filters
			poMountFilter = new MountFilter;
			
			// set data from config
			poMountData = poVolumeConfig->getMountData();
			
			// set filter data
			poMountFilter->setSrcData(poMountData);
			poMountFilter->setBasePathSrc(poMountData);
			
			// add filter to pline
			addFilter(poMountFilter);
			
			bPlineCreated = true;
			
			break;
		case kMountLoop:
			// new filters
			poLoopFilter = new LoopFilter;
			poMountFilter = new MountFilter;
			
			// set data from config
			poLoopData = (LoopData*)poVolumeConfig->getDataPtr();
			poMountData = poVolumeConfig->getMountData();
			
			// set filter data
			poLoopFilter->setSrcData(poLoopData);
			poLoopFilter->setDestData(poMountData);
			
			poMountFilter->setSrcData(poMountData);
			poMountFilter->setBasePathSrc(poLoopData);
			
			// add filter to pline
			addFilter(poLoopFilter);
			addFilter(poMountFilter);
			
			bPlineCreated = true;
			
			break;
		case kMountCrypto:
			// check data
			if (!poVolumeConfig->getCryptData())
				break;

			// new filters
			poCryptFilter = new CryptFilter;
			poMountFilter = new MountFilter;
			
			// set data from config
			poCryptData = poVolumeConfig->getCryptData();
			poMountData = poVolumeConfig->getMountData();
			
			// set filter data
			poCryptFilter->setSrcData(poCryptData);
			poCryptFilter->setDestData(poMountData);
			
			poMountFilter->setSrcData(poMountData);
			poMountFilter->setBasePathSrc(poCryptData);
			
			// add filter to pline
			addFilter(poCryptFilter);
			addFilter(poMountFilter);
			
			bPlineCreated = true;
			
			break;
		case kMountLCrypto:
			// check data
			if (!poVolumeConfig->getCryptData())
				break;
				
			// new filters
			poLoopFilter = new LoopFilter;
			poCryptFilter = new CryptFilter;
			poMountFilter = new MountFilter;
			
			// set data from config
			poLoopData = (LoopData*)poVolumeConfig->getDataPtr();
			poCryptData = poVolumeConfig->getCryptData();
			poMountData = poVolumeConfig->getMountData();
			
			// set filter data
			poLoopFilter->setSrcData(poLoopData);
			poLoopFilter->setDestData(poCryptData);
			
			poCryptFilter->setSrcData(poCryptData);
			poCryptFilter->setDestData(poMountData);
			
			poMountFilter->setSrcData(poMountData);
			poMountFilter->setBasePathSrc(poLoopData);
			
			// add filter to pline
			addFilter(poLoopFilter);
			addFilter(poCryptFilter);
			addFilter(poMountFilter);
			
			bPlineCreated = true;
			
			break;
		default:
			break;
	}
	
	// 
	if (bPlineCreated)
	{
		assert(poMountData);
		
		// this must be always created
		// as user can change poMounteData->getDirSetPerm at run time
		gpoPermFilter = new PermFilter;
	
		gpoPermFilter->setSrcData(poMountData);
		
		gpoTrashFilter = new TrashFilter;
		
		gpoTrashFilter->setSrcData(poMountData);
		
		addFilter(gpoPermFilter);
		addFilter(gpoTrashFilter);
	}
}

//==============================================================================

bool MountPline::dumpStatus()
{
	if (!setFiltersRunMode(MountBaseFilter::kRunProbe))
		return false;
	
	gvecDumpStatus.clear();
	
	gbDumpStatus = true;
	
	geCurrRunMode = kRunDumpStatus;
	
	Pline::run();
	
	gbDumpStatus = false;
	
	return true;
}

//==============================================================================

bool MountPline::eject()
{
	bool bPlineRun = false;

	debug_message(  DEBUG_PLINE, "Src[%s] ejecting.", 
					gpoVolumeConfig->getData().getSrc().c_str());
	
	if (!setFiltersRunMode(MountBaseFilter::kRunDetach))
		return false;

	// reverse filters
	reverseFilters();

	geCurrRunMode = kRunEject;
	bPlineRun = Pline::run();

	// re-order filters
	reverseFilters();
	
	initMountStatus();
	
	return bPlineRun;
}

//==============================================================================

std::vector<MountPline::tDumpStatus>& MountPline::getDumpStatus()
{
	return gvecDumpStatus;
}

//==============================================================================

MountPline::eMountType MountPline::getMountType(VolumeConfig *poVolumeConfig)
{
	eMountType eMountTypeItem = kNoMount;
	Glib::ustring stSrc;
	bool bSrcIsFile = false;
	struct stat statbuf;
	
	stSrc = poVolumeConfig->getData().getSrc();
	std::string st = stSrc.c_str();
	
	if (stSrc.empty() || stat(stSrc.c_str(), &statbuf) != 0)
		return kNoMount;
		
	if (S_ISBLK(statbuf.st_mode)) // src is block dev
		bSrcIsFile = false;
	else if (S_ISREG(statbuf.st_mode)) // src is file
		bSrcIsFile = true;
	else
		return kNoMount;
	
	if (poVolumeConfig->getCryptData()) // encrypted volume
		eMountTypeItem = (bSrcIsFile) ? kMountLCrypto : kMountCrypto;
	else
		eMountTypeItem = (bSrcIsFile) ? kMountLoop : kMountSimple;
	
	return eMountTypeItem;
}

//==============================================================================

void MountPline::initDiskUsage()
{
	gtDiskUsage.bValid = false;
	gtDiskUsage.stDiskSize.clear();
	gtDiskUsage.stSpaceUsed.clear();
	gtDiskUsage.stSpaceAvailable.clear();
	gtDiskUsage.iUsedSpacePct = 0;
}

//==============================================================================

void MountPline::initMountStatus()
{
	gstFSys.clear();
	gbEncrypted = false;
	gstCipher.clear();
	
	gbHasProbedFilters = false;
	gbProbeError = false;
}

//==============================================================================

void MountPline::insertDumpItem(Filter* poFilter)
{
	tDumpStatus tDumpStatusItem;
			
	tDumpStatusItem.stFilterName = poFilter->getFilterName();
	tDumpStatusItem.stSrc = poFilter->getSrcData()->getSrc();
	tDumpStatusItem.stSrcType = _("Source");
	
	gvecDumpStatus.push_back(tDumpStatusItem);
				 
	if (poFilter->getFilterName() == _("Mount"))
	{
		tDumpStatusItem.stFilterName = poFilter->getFilterName();
		tDumpStatusItem.stSrc = ((MountData*)poFilter->getSrcData())->getDir();
		tDumpStatusItem.stSrcType = _("Directory");
		
		gvecDumpStatus.push_back(tDumpStatusItem);
	}
}

//==============================================================================

bool MountPline::isInUse(bool *pbMounted)
{
	eRunMode eRunModePrev = geRunMode; // save run mode status

	geRunMode = kRunProbe;

	run();
	
	if (gbHasProbedFilters && pbMounted)
		*pbMounted = !gbProbeError;
	else
	{
		if (pbMounted)
			*pbMounted = false;
	}
	
	geRunMode = eRunModePrev; // reset run mode status
	
	return (gbHasProbedFilters) ? true : false;
}

//==============================================================================

bool MountPline::mount()
{
	debug_message(  DEBUG_PLINE, "Src[%s] mounting.", 
					gpoVolumeConfig->getData().getSrc().c_str());

	initMountStatus();
	
	if (setFiltersRunMode(MountBaseFilter::kRunAttach))
	{
		geCurrRunMode = kRunMount;
		return Pline::run();
	}
	else
		return false;
}

//==============================================================================

bool MountPline::onFilterPreRun(Filter *poFilter)
{
	if (poFilter == gpoPermFilter)
		return (geCurrRunMode == kRunMount && geLastRunMode == kRunMount);
	else if (poFilter == gpoTrashFilter)
		return (geCurrRunMode == kRunEject && geLastRunMode == kRunEject);

	MountBaseFilter* poMountFilter = (MountBaseFilter*)poFilter;
	
	if (poMountFilter->getRunMode() == MountBaseFilter::kRunAttach)
		return (!poMountFilter->isAttached());
	else if (poMountFilter->getRunMode() == MountBaseFilter::kRunDetach)
		return (poMountFilter->isAttached());
	else if (poMountFilter->getRunMode() == MountBaseFilter::kRunProbe)
	{
		// override pline::run what would interrupt the pline if the
		// filter returns errors
		onFilterRun(poMountFilter, !(poMountFilter->run()));
			
		return false;
	}
		
	return true;
}

//==============================================================================

bool MountPline::onFilterRun(Filter *poFilter, bool bHasRunWithErrors)
{
	if (poFilter == gpoPermFilter
		|| poFilter == gpoTrashFilter)
	{
		return Pline::onFilterRun(poFilter, bHasRunWithErrors);
	}
	
	MountBaseFilter* poMountFilter = (MountBaseFilter*)poFilter;
	
	if (bHasRunWithErrors)
	{
		if (poMountFilter->getLastRunMode() == MountBaseFilter::kRunProbe)
			gbProbeError = true;
		
		return Pline::onFilterRun(poFilter, bHasRunWithErrors);
	}
				
	if (poMountFilter->getLastRunMode() == MountBaseFilter::kRunProbe)
	{
		gbHasProbedFilters = true;
		
		if (gbDumpStatus)
			insertDumpItem(poMountFilter);
	}
		
	// set gstFSys
	if (poMountFilter->getLastRunMode() == MountBaseFilter::kRunProbe
		|| poMountFilter->getLastRunMode() == MountBaseFilter::kRunAttach) 
	{
		if (poMountFilter->getFilterName() == "Mount")
		{
			MountData *poMountData = (MountData*)poMountFilter->getSrcData();
			
			if (poMountData)
			{
				gstFSys = poMountData->getFSys();
				
				// save mount point and permissions
				if (goVolumeConfig.getMountData())
				{
					if (poMountFilter->getLastRunMode() == MountBaseFilter::kRunAttach)
					{
						goVolumeConfig.getMountData()->setDir(
							poMountData->getDir());
					
						goVolumeConfig.getMountData()->setDirOwner(
							poMountData->getDirOwner());
						goVolumeConfig.getMountData()->setDirGroup(
							poMountData->getDirGroup());
						goVolumeConfig.getMountData()->setDirPerm(
							poMountData->getDirPerm());
						goVolumeConfig.getMountData()->setDirSetPerm(
							poMountData->getDirSetPerm());
					}
				}
			}
		}
	}
	
	if (poMountFilter->getFilterName() == "Crypt")
	{
		gbEncrypted = true;
		
		if (poMountFilter->getLastRunMode() == MountBaseFilter::kRunProbe
			|| poMountFilter->getLastRunMode() == MountBaseFilter::kRunAttach)
		{
			CryptFilter* poCryptFilter = (CryptFilter*)poMountFilter;
			
			Glib::ustring stCipher;
			Glib::ustring stKeySize;
			
			
			if (poCryptFilter->getDestData()
				&& CryptUtil::getStatus(poCryptFilter->getDestData()->getSrc(), &stCipher, &stKeySize))
			{
				gstCipher = Util::compose(	_("Cipher: %s, Key Size: %s"), 
											stCipher.c_str(), 
											stKeySize.c_str());
			}
		}
	}
	
	return Pline::onFilterRun(poFilter, bHasRunWithErrors);
}

//==============================================================================
	
bool MountPline::probe()
{
	debug_message(  DEBUG_PLINE, "Src[%s] probing.", 
					gpoVolumeConfig->getData().getSrc().c_str());
	
	initMountStatus();
	
	bool bRun = false;
	
	if (setFiltersRunMode(MountBaseFilter::kRunProbe))
	{
		geCurrRunMode = kRunProbe;
		bRun = Pline::run();
	}
		
	return bRun;
}

//==============================================================================

bool MountPline::run()
{
	eRunMode eRunModeItem = geRunMode;
	bool bRet = false;
		
	// manage completed signal locally
	Pline::skipCompletedSignal(true);
	
	// set run mode to no mode
	geRunMode = kNoRunMode;
	geLastRunMode = eRunModeItem;
	
	// init others
	gbHasProbedFilters = false;
	gbDumpStatus = false;
	
	if (eRunModeItem == kNoRunMode)
	{
		bRet = false;
		
		m_signal_completed(bRet);
		
		return bRet;
	}
	else if (eRunModeItem == kRunDiskUsage)
		bRet = probe() ? calcDiskUsage() : false;
	else if (eRunModeItem == kRunDumpStatus)
		bRet = probe() ? dumpStatus() : false;
	else if (eRunModeItem == kRunEject)
	{
		updateVolumeConfigPline(&goVolumeConfig);
		
		bRet = (probe() || gbHasProbedFilters) ? eject() : false;
		
		// save error if needed
		tError tLastError = gtLastError;
		
		// dump status
		dumpStatus();
		
		// restore error
		gtLastError = tLastError;
		
		m_signal_completed(bRet);
	}
	else if (eRunModeItem == kRunMount)
	{
		Pline::removeFilters();
				
		*gpoVolumeConfig = goVolumeConfig;
			
		createVolumeConfigPline(gpoVolumeConfig);

		if (mount())
		{
			calcDiskUsage();
			dumpStatus();
			
			bRet = true;
		}
		else
		{
			tError tMountError = gtLastError;
			
			eject();
			
			dumpStatus();
			
			gtLastError = tMountError;
			
			bRet = false;
		}
		
		m_signal_completed(bRet);
	}
	else if (eRunModeItem == kRunProbe)
		bRet = probe();
		
	return bRet;
}

//==============================================================================

bool MountPline::setFiltersRunMode(MountBaseFilter::eRunMode eRunModeItem)
{
	std::vector<Filter*> vecFilter = Pline::getFilters();
	
	Pline::initError();
	
	if (vecFilter.size() == 0)
	{
		gtLastError.bIsFilterError = false;
		gtLastError.stErrMsg = _("Device not recognized.");
		
		// test src
		Glib::ustring stSrc = gpoVolumeConfig->getData().getSrc();
		
		Util::isFileExist(stSrc, &gtLastError.stErrMsg);
		
		return false;
	}
		
	for (   std::vector<Filter*>::size_type xi = 0;
			xi < vecFilter.size();
			++xi)
	{
		if (vecFilter[xi] == gpoPermFilter
			|| vecFilter[xi] == gpoTrashFilter)
		{
			continue;
		}
			
		MountBaseFilter *poFilter = (MountBaseFilter*)vecFilter[xi];
		
		poFilter->setRunMode(eRunModeItem);
	}
	
	return true;
}

//==============================================================================

void MountPline::updatePline()
{
	Pline::removeFilters();
				
	*gpoVolumeConfig = goVolumeConfig;
			
	createVolumeConfigPline(gpoVolumeConfig);
}

//==============================================================================

void MountPline::updateVolumeConfigPline(VolumeConfig *poVolumeConfig)
{
	if (gpoVolumeConfig->getMountData()
		&& poVolumeConfig->getMountData())
	{
		gpoVolumeConfig->getMountData()->setEmptyTrash(
			poVolumeConfig->getMountData()->isEmptyTrash());
	
		gpoVolumeConfig->getMountData()->setEmptyTrashConfirm(
			poVolumeConfig->getMountData()->isEmptyTrashConfirm());
	}
}

//==============================================================================


} // Pline
