/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef QDEBUG
#define FILENAME_ROOT	".emount.cnf"
#else
#define FILENAME_ROOT	".emount.debug.cnf"
#endif

#include "QConfig.h"
#include "QDebug.h"
#include "QConfigDefaultConfig.h"
#include "Util.h"
#include "../config.h"

#include <vector>
#include <iostream>

//==============================================================================

QConfig::QConfig()
{
	gbSaveBackup = false;
	
	// set configuration filename
	gstConfigFilename = Glib::build_filename(g_get_home_dir(), FILENAME_ROOT);
	
	loadConfig();
}

//==============================================================================

QConfig::~QConfig()
{
	// save backup if file structure has been modified
	// this should happen only when QCONF_VER changes
	if (gbSaveBackup)
	{
		Glib::ustring stBackupData;
		
		if (Util::file2string(gstConfigFilename, &stBackupData))
		{
			bool bIsValidBackupFilename = false;
			Glib::ustring stBackupFilename;
			
			for (int i=0; i<256; ++i)
			{
				stBackupFilename = Util::compose(	"%s.%d.bak", 
													gstConfigFilename.c_str(), 
													i);
				
				if (!Util::isFileExist(stBackupFilename))
				{
					bIsValidBackupFilename = true;
					break;
				}
			}
			
			if (bIsValidBackupFilename)
				Util::string2file(stBackupData, stBackupFilename);
		}
	}
	
	saveConfig();
	
	// delete VolumeConfig
	for (   std::list<VolumeConfig*>::iterator ite = glstVolumeConfig.begin();
			ite != glstVolumeConfig.end();
			++ite)
	{
		delete *ite;
	}
}

//==============================================================================

bool QConfig::addVolume(VolumeConfig* poVolumeConfig)
{
	if (!poVolumeConfig)
		return false;
	
	// check volume label is unique
	for (   std::list<VolumeConfig*>::iterator ite = glstVolumeConfig.begin();
			ite != glstVolumeConfig.end();
			++ite)
	{
		if (((VolumeConfig*)*ite)->getData().getLabel() ==
				poVolumeConfig->getData().getLabel())
		{
			return false;
		}
	}
	
	glstVolumeConfig.push_back(poVolumeConfig);
	
	return true;
}

//==============================================================================

bool QConfig::findVolume(   const Glib::ustring& stLabel,
							VolumeConfig& oVolumeConfig)
{
	for (   std::list<VolumeConfig*>::iterator ite = glstVolumeConfig.begin();
			ite != glstVolumeConfig.end();
			++ite)
	{
		if (((VolumeConfig*)*ite)->getData().getLabel() == stLabel)
		{
			oVolumeConfig = *((VolumeConfig*)*ite);
			
			return true;
		}
	}
	
	return false;
}
					
//==============================================================================

QConfig& QConfig::getInstance()
{
	static QConfig oConfig;
	
	return oConfig;
}

//==============================================================================

bool QConfig::getMainWinH(int *pi)
{
	try
	{
		*pi = goKeyFile.get_integer("Main", "WinSizeY");
					
		return true;
	}
	catch(...)
	{
		return false;
	}
}

//==============================================================================

bool QConfig::getMainWinW(int *pi)
{
	try
	{
		*pi = goKeyFile.get_integer("Main", "WinSizeX");
		
		return true;
	}
	catch(...)
	{
		return false;
	}
}

//==============================================================================

bool QConfig::getMainWinX(int *pi)
{
	try
	{
		*pi = goKeyFile.get_integer("Main", "WinPosX");
		
		return true;
	}
	catch(...)
	{
		return false;
	}
}

//==============================================================================

bool QConfig::getMainWinY(int *pi)
{
	try
	{
		*pi = goKeyFile.get_integer("Main", "WinPosY");
		
		return true;
	}
	catch(...)
	{
		return false;
	}
}

//==============================================================================

bool QConfig::getSortBy(int *pi)
{
	try
	{
		*pi = goKeyFile.get_integer("Main", "SortBy");
		
		return true;
	}
	catch(...)
	{
		return false;
	}
}

//==============================================================================

bool QConfig::isDisableAutomount()
{
	try
	{
		bool b = goKeyFile.get_boolean("Main", "DisableAutomount");
		
		return b;
	}
	catch(...)
	{
		return false;
	}
}

//==============================================================================

bool QConfig::isHideToolTips()
{
	try
	{
		bool b = goKeyFile.get_boolean("Main", "HideToolTips");
		
		return b;
	}
	catch(...)
	{
		return false;
	}
}
			
//==============================================================================

bool QConfig::isSaveWinPos()
{
	try
	{
		bool b = goKeyFile.get_boolean("Main", "WinPos");
		
		return b;
	}
	catch(...)
	{
		return false;
	}
}

//==============================================================================

bool QConfig::isSaveWinSize()
{
	try
	{
		bool b = goKeyFile.get_boolean("Main", "WinSize");
		
		return b;
	}
	catch(...)
	{
		return false;
	}
}

//==============================================================================

bool QConfig::loadConfig()
{
	Glib::ustring stData;
	
	// load file into string
	if (Util::file2string(gstConfigFilename, &stData))
	{
		debug_message(DEBUG_APP, "using root configuration file: %s", gstConfigFilename.c_str());
	}
	else
	{
		// file doesn't exist
		stData = loadDefaultConfig();
		
		debug_message(DEBUG_APP, "failed opening root configuration file: [%s] using default configuration", gstConfigFilename.c_str());
	}

	// convert v1.0.0
	if (stData.find("QConfVer=1.0.0") != std::string::npos)
	{
		// doesn't include a main group
		if (stData.find("[Main]") == std::string::npos)
			stData.insert(0, "[Main]\n");
		
		gbSaveBackup = true;
	}
	
	// load goKeyFile
	try
	{
		goKeyFile.load_from_data(   stData, 
									(Glib::KeyFileFlags)(G_KEY_FILE_NONE | G_KEY_FILE_KEEP_COMMENTS));		
	}
	catch(...)
	{
		std::cerr << "Failed loading configuration." << std::endl;
		return false;
	}
	
	// load Volumes
	try
	{
		std::vector<Glib::ustring> vecGroup = goKeyFile.get_groups();
		
		for (   std::vector<Glib::ustring>::size_type xi = 0;
				xi < vecGroup.size();
				++xi)
		{
			// Volume config
			if (vecGroup[xi].find("DevConf/") == 0)
			{
				VolumeConfig *poVolumeConfig = new VolumeConfig;
				
				if (poVolumeConfig->loadConfig(vecGroup[xi], goKeyFile))
					glstVolumeConfig.push_back(poVolumeConfig);
				else
					std::cerr << "Failed loading Volume: " << vecGroup[xi] << std::endl;
			}
			else if (vecGroup[xi].find("DevConf") == 0)
			{
				Glib::ustring stHomeDir = Glib::get_home_dir();
				
				gstDfDir = 
					(goKeyFile.has_key(vecGroup[xi], "DestDir")) ? 
						goKeyFile.get_string(vecGroup[xi], "DestDir") 
							: stHomeDir;
							
				gstDfFSys = 
					(goKeyFile.has_key(vecGroup[xi], "FSys")) ? 
						goKeyFile.get_string(vecGroup[xi], "FSys")
							: "ext3";
							
				gstDfSrc = 
					(goKeyFile.has_key(vecGroup[xi], "Src")) ? 
						goKeyFile.get_string(vecGroup[xi], "Src")
							: stHomeDir;
				
				gstDfKey = 
					(goKeyFile.has_key(vecGroup[xi], "Key")) ? 
						goKeyFile.get_string(vecGroup[xi], "Key")
							: stHomeDir;
				
				gstDfCloneDir = 
					(goKeyFile.has_key(vecGroup[xi], "CloneDest")) ? 
						goKeyFile.get_string(vecGroup[xi], "CloneDest")
							: stHomeDir;
			}
		}
	}
	catch(...)
	{
		std::cerr << "Failed loading volumes configuration." << std::endl;
		return false;
	}
	
	return true;
}

//==============================================================================

Glib::ustring QConfig::loadDefaultConfig() const
{
	return QCONFIG_DEFAULTCONFIG_STRING;
}

//==============================================================================

void QConfig::removeVolume(VolumeConfig* poVolumeConfig)
{
	for (   std::list<VolumeConfig*>::iterator ite = glstVolumeConfig.begin();
			ite != glstVolumeConfig.end();
			++ite)
	{
		if (((VolumeConfig*)*ite) == poVolumeConfig)
		{
			glstVolumeConfig.remove(*ite);
			break;
		}
	}
	
	poVolumeConfig->removeConfig(goKeyFile);	
	delete poVolumeConfig;
}

//==============================================================================

bool QConfig::saveConfig()
{
	Glib::ustring stData;
	
	if (gstConfigFilename.empty())
		return false;
		
	try
	{	
		// save VolumeConfig
		for (   std::list<VolumeConfig*>::iterator ite = glstVolumeConfig.begin();
				ite != glstVolumeConfig.end();
				++ite)
		{
			((VolumeConfig*)*ite)->saveConfig(goKeyFile);
		}
		
		// set config version
		goKeyFile.set_string("Main", "QConfVer", QCONF_VER);
		
		// set default data
		goKeyFile.set_string("DevConf", "DestDir", gstDfDir);
		goKeyFile.set_string("DevConf", "FSys", gstDfFSys);
		goKeyFile.set_string("DevConf", "Src", gstDfSrc);
		goKeyFile.set_string("DevConf", "Key", gstDfKey);
		goKeyFile.set_string("DevConf", "CloneDest", gstDfCloneDir);
		
		// save Data
		stData = goKeyFile.to_data();

		if (!Util::string2file(stData, gstConfigFilename))
		{
			std::cerr << "Failed saving configuration." << std::endl;
			return false;
		}
		else
		{
			debug_message(DEBUG_APP, "root configuration file saved");
		}
	}
	catch(...)
	{
		std::cerr << "Failed saving configuration." << std::endl;
		return false;
	}
	
	return true;
}

//==============================================================================

void QConfig::setDisableAutomount(const bool bDisable)
{
	try
	{
		goKeyFile.set_boolean("Main", "DisableAutomount", bDisable);
	}
	catch(...)
	{
	}
}

//==============================================================================

void QConfig::setHideToolTips(const bool bHide)
{
	try
	{
		goKeyFile.set_boolean("Main", "HideToolTips", bHide);
	}
	catch(...)
	{
	}
}

//==============================================================================

void QConfig::setSaveWinPos(const bool bSave)
{
	try
	{
		goKeyFile.set_boolean("Main", "WinPos", bSave);
	}
	catch(...)
	{
	}
}

//==============================================================================

void QConfig::setSaveWinSize(const bool bSave)
{
	try
	{
		goKeyFile.set_boolean("Main", "WinSize", bSave);
	}
	catch(...)
	{
	}
}

//==============================================================================

void QConfig::setMainWinX(const int i)
{
	try
	{
		goKeyFile.set_integer("Main", "WinPosX", i);
	}
	catch(...)
	{
	}
}

//==============================================================================

void QConfig::setMainWinY(const int i)
{
	try
	{
		goKeyFile.set_integer("Main", "WinPosY", i);
	}
	catch(...)
	{
	}
}

//==============================================================================

void QConfig::setMainWinW(const int i)
{
	try
	{
		goKeyFile.set_integer("Main", "WinSizeX", i);
	}
	catch(...)
	{
	}
}

//==============================================================================

void QConfig::setMainWinH(const int i)
{
	try
	{
		goKeyFile.set_integer("Main", "WinSizeY", i);
	}
	catch(...)
	{
	}
}

//==============================================================================

void QConfig::setSortBy(const int i)
{
	try
	{
		goKeyFile.set_integer("Main", "SortBy", i);
	}
	catch(...)
	{
	}
}

//==============================================================================
